************************************************************************	
	PROGRAM DRIVER						         
*  Desciption:                                                         *
*       Driver program to test BOXCQP                                  *	
*                                                                      *
*  Implemented by:                                                     *
*         C. Voglis                                                    *
*         Dept. of Computer Science, University of Ioannina, GREECE.   *
*         University of Ioannina, GREECE.                              *
*         Email: voglis@cs.uoi.gr                                      *
*                                                                      *
*         I. E. Lagaris                                                *
*         Dept. of Computer Science, University of Ioannina, GREECE.   *
*         Email: lagaris@cs.uoi.gr                                     *
************************************************************************	
	IMPLICIT DOUBLE PRECISION (A-H, O-Z)
C
C  Parameter the contains the maximum dimension N.
	PARAMETER (MAXN=6500)
C
C  Local variables.
	DIMENSION POINT(MAXN)
	DIMENSION B(MAXN*MAXN), D(MAXN), XR(MAXN), XL(MAXN), DM(MAXN)
C
C  Work arrays. 
	DIMENSION AUX(MAXN*MAXN), W1(MAXN), W2(MAXN), W3(MAXN)
	DIMENSION W4(MAXN), W5(MAXN+1), W6(MAXN), W7(MAXN)
	DIMENSION W8(MAXN), W9(MAXN)

	COMMON /CMACHE/EPS  
C
C Time arrays 
	dimension diff(10), fval(10)
C
C  Character array.
	CHARACTER*20 FNAME
	CHARACTER*40 MESS
C
C  Input the name of the file that contain the input data.
C  B(N, N), D(N), XR(N), XL(N)
	WRITE (*, *) 'Input file name: '
	READ (*, *) FNAME
	OPEN (UNIT=8,FILE=FNAME,STATUS='OLD',ERR=17)
C
C  Calculate machine accuracy
	CALL MACACC (COMACC)
	EPS = COMACC
C
C  Read the dimensionality of the problem.
	READ(8, *) N	
C
C  Read the Hessian matrix B(N, N). (Column order)
	DO 10 I=1, N
	 DO 20 J=1, N 
	  READ(8, *) B(J + N * (I-1))
20       CONTINUE
10	CONTINUE
C
C  Read linear term vector D(N).  
	DO 30 I=1, N
	 READ (8, *) D(I)
30	CONTINUE
C
C  Read upper bound XR(N).
	DO 40 I=1, N
	 READ (8, *) XR(I)
40	CONTINUE
C
C  Read lower  bound XL(N).
	DO 50 I=1, N
	 READ (8, *) XL(I)
50	CONTINUE
	CLOSE(8)

C
C  Call the minimizer. Modified Cholesky
C 
C     Define input parameters. 
	IQP = 0
	IPR = 0
	INP = 1
	CALL CPU_TIME (TIMER1)

	CALL  BOXCQP(B,POINT,D,XL,XR,AUX,N,ITRCNT,IQP,INP,INFO,COMACC,
     >			IPR, W1, W2, W3, W4, W5, W6, W7, W8, W9)	
	
	CALL CPU_TIME (TIMER2)
	diff(1) = TIMER2-TIMER1
	fval(1) = QUADRATIC(B, D, POINT, N)
	write (*, *) 'Sol: ', (POINT(K), K=1, N)
	write (*, *) QUADRATIC(B, D, POINT, N)
	write (*, *) 'Time: ',  diff(1)
	STOP
 17	WRITE (*, *) 'END OF FILE'
 14     FORMAT(1X, 6(1X, F14.4))
	
	END 



	SUBROUTINE TRASPROB(B, BBAR, D, DBAR, XR, XL, WORK, N)
	IMPLICIT DOUBLE PRECISION (A-H, O-Z)
	DIMENSION B(N, N), BBAR(N, N), WORK(N) 
	DIMENSION D(N), DBAR(N), XR(N), XL(N)

	DO 9 I=1, N
	 WORK(I) = 0.5*(XR(I) - XL(I))	 
9       CONTINUE
	
	DO 10 I=1, N
	 DO 20 J=1, N
	   BBAR(I, J) = WORK(I)*B(I, J)*WORK(J)	 
20     CONTINUE	 
10    CONTINUE


	DO 30 I=1, N
	 DBAR(I) = 0.0
	 DO 40 J=1, N
	  DBAR(I) = DBAR(I) + (B(I, J) * 0.5 * (XL(J) + XR(J)))
40     CONTINUE	
	 DBAR(I) = WORK(I)*(D(I) - DBAR(I)) 
30    CONTINUE

	

	END

	SUBROUTINE INVTRANS(XR, XL, P, PBAR, N)
	IMPLICIT DOUBLE PRECISION (A-H, O-Z)
	DIMENSION XL(N), XR(N), P(N), PBAR(N)

	DO 10 I=1, N
       P(I) = 0.5*(XL(I) + XR(I)) + (0.5*(XR(I)-XL(I)) * PBAR(I))
10    CONTINUE	

	END
